#!/bin/bash

# Purpose of script: Generates an argumentation framework (AF) and subsequently constructs a DIMACS such that the models of the formula correspond to complete extensions of the AF
# Parameters of generation: number of arguments in the AF, expected edge density (prob. that between two distinct arguments is an attack) and a random seed

# variables
EXPECTEDDENS=${EXPECTEDDENS:='0.1'}
GENERATOR=${GENERATOR:='GenerateGraphTreeWidth'}
MYTMPDIR=${MYTMPDIR:=''}
NUMBEROFARGS=${NUMBEROFARGS:='5'}
NUMBEROFCLAUSES=${NUMBEROFCLAUSES:='0'}
NUMBEROFVARS=${NUMBEROFVARS:='10'}
NUMPATTERN=${NUMPATTERN:=''}
PARSER=${PARSER:='parseAFcomp'}
RANDOMSEED=${RANDOMSEED:='555'}
TMPDAF=${TMPDAF:=''}
TMPDIMACS=${TMPDIMACS:=''}
TMPERR=${TMPERR:=''}
WORKINGDIR=${WORKINGDIR:=''}

# usage
usage() {
echo "usage: "
echo $0 "numberOfArgs expectedEdgeDensity randomSeed"
echo "numberOfArgs        ... Number of arguments (vertices) in the argumentation framework (directed graph)"
echo "expectedEdgeDensity ... probability that there is an attack (directed edge) from A to B in the AF"
echo "randomSeed          ... random seed for generation. (using Java Random class)"
echo ""
echo "Example call: " $0 " 100 0.1 555"
echo ""
echo "instance AF is generated in ASP format: (one per line)"
echo "           arg(x).   ... x is an argument"
echo "           att(x,y). ... x attacks y"
}

# create a temporary directory, in which we create some temporary files (which should be removed after termination)
MYTMPDIR=$(mktemp -d)
trap 'rm -rf "$MYTMPDIR"' EXIT INT TERM HUP

# temporary files
TMPDIMACS=`mktemp -p $MYTMPDIR`
TMPERR=`mktemp -p $MYTMPDIR`
TMPDAF=`mktemp -p $MYTMPDIR`

# current working directory
WORKINGDIR=`dirname $0`

# arguments
NUMBEROFARGS=$1
EXPECTEDDENS=$2
RANDOMSEED=$3

# check usage
NUMPATTERN='^[0-9]+$'
if ! [[ $NUMBEROFARGS =~ $NUMPATTERN ]] ; then
   echo "error: First argument is not an integer" >&2; 
   usage
   exit 1
fi

NUMPATTERN='^[0-9].[0-9]$'
if ! [[ $EXPECTEDDENS =~ $NUMPATTERN ]] ; then
   echo "error: Second argument must be between 0 and 1. Formatted as N.N" >&2; 
   usage
   exit 1
fi

NUMPATTERN='^[0-9]+$'
if ! [[ $RANDOMSEED =~ $NUMPATTERN ]] ; then
   echo "error: Third argument is not an integer" >&2; 
   usage
   exit 1
fi

# check if java generator is compiled
if ! [[ -e $GENERATOR.class ]]; then
   echo "error: Java generator for AFs not compiled (eg run javac *.java)"
fi

# run AF generator
java $GENERATOR $NUMBEROFARGS $EXPECTEDDENS $RANDOMSEED > $TMPDAF

if [[ "$?" != "0" ]];then
   echo "error: AF generator did not terminate properly."
   exit 1
fi

# parse instance to DIMACs (map indicates mapping between arguments and DIMACS variables)
$WORKINGDIR/$PARSER $TMPDAF > $TMPDIMACS 2> $TMPERR
grep -v "map" $TMPERR >> $TMPDIMACS

# get number of vars/clauses
NUMBEROFCLAUSES=`wc -l $TMPDIMACS | awk '{print $1}'`
NUMBEROFVARS=$[$NUMBEROFARGS*2]

# output AF, mappings and DIMACS
echo "c argumentation framework generated with "$NUMBEROFARGS" arguments and expected edge density of "$EXPECTEDDENS" using generator "$0
echo "c random seed: "$RANDOMSEED
echo "c DIMACS is SATISFIABLE" 
echo "c generator call: "$0 $*
echo "c generated AF"
cat $TMPDAF | awk '{print "c "$1}'
echo "c mappings from arguments to propositional variables [map(arg,var)]"
grep "map" $TMPERR | awk '{print "c "$1}'
echo "p cnf "$NUMBEROFVARS $NUMBEROFCLAUSES > $TMPDIMACS.new && cat $TMPDIMACS >> $TMPDIMACS.new
cat $TMPDIMACS.new
